// Based on the work by DFRobot
// Added some stack improvements by Z-Wave>ME especially for "pjpankhurst"
#include "ZUNO_LCD_I2C.h"

// It's will be clearly to place all specific data here
// commands
#define LCD_CLEARDISPLAY 0x01
#define LCD_RETURNHOME 0x02
#define LCD_ENTRYMODESET 0x04
#define LCD_DISPLAYCONTROL 0x08
#define LCD_CURSORSHIFT 0x10
#define LCD_FUNCTIONSET 0x20
#define LCD_SETCGRAMADDR 0x40
#define LCD_SETDDRAMADDR 0x80

// flags for display entry mode
#define LCD_ENTRYRIGHT 0x00
#define LCD_ENTRYLEFT 0x02
#define LCD_ENTRYSHIFTINCREMENT 0x01
#define LCD_ENTRYSHIFTDECREMENT 0x00

// flags for display on/off control
#define LCD_DISPLAYON 0x04
#define LCD_DISPLAYOFF 0x00
#define LCD_CURSORON 0x02
#define LCD_CURSOROFF 0x00
#define LCD_BLINKON 0x01
#define LCD_BLINKOFF 0x00

// flags for display/cursor shift
#define LCD_DISPLAYMOVE 0x08
#define LCD_CURSORMOVE 0x00
#define LCD_MOVERIGHT 0x04
#define LCD_MOVELEFT 0x00



// flags for backlight control
#define LCD_BACKLIGHT 0x08
#define LCD_NOBACKLIGHT 0x00

#define En B00000100  // Enable bit
#define Rw B00000010  // Read/Write bit
#define Rs B00000001  // Register select bit
// Moved it to global scope
// It's a "workaround".
// Mangling fails on long names. It will be fixed in 2.1.2 
byte g_lcd_row_offsets[4] = { 0x00, 0x40, 0x14, 0x54 };

// Global variables to translate parameters of axilary functions
// --------------------------------------------------------------
extern byte g_lcd_aux_addr; 
extern byte g_lcd_aux_data; 
extern byte g_lcd_aux_tmp;
byte g_lcd_aux_addr;
byte g_lcd_aux_data;
byte g_lcd_aux_tmp;
// --------------------------------------------------------------

// Add two functions with minimum number of 
// parameters to minimize stack usage
void lcd_aux_expander_write()
{
	Wire.beginTransmission(g_lcd_aux_addr);
	Wire.write(g_lcd_aux_data);
	Wire.endTransmission();
}
void lcd_aux_write4bits()
{
	lcd_aux_expander_write();	
	g_lcd_aux_tmp = g_lcd_aux_data;
	g_lcd_aux_data |= En;
	delayMicroseconds(1);
	lcd_aux_expander_write();
	g_lcd_aux_data = g_lcd_aux_tmp;
	g_lcd_aux_data &= ~En;
	lcd_aux_expander_write();
	delayMicroseconds(50);
}
// Use the macroses instead of function
// It will be like inline functions
// It decorates global varriables usage
// It's a bad idea to use private methods instaed of this - you will kill the stack
#define lcd_select_addr() g_lcd_aux_addr = _Addr
#define lcd_expander_write(value)\
		g_lcd_aux_data = value;\
		lcd_aux_expander_write();
#define lcd_write4bits(value)\
		g_lcd_aux_data = value;\
		lcd_aux_write4bits();
#define lcd_send(value, mode) \
		lcd_select_addr();\
		g_lcd_aux_data = value; \
		g_lcd_aux_data &= 0xf0; \
		g_lcd_aux_data |= mode; \
		lcd_aux_write4bits();   \
		g_lcd_aux_data = value; \ 
		g_lcd_aux_data <<= 4;   \
		g_lcd_aux_data &= 0xf0; \
		g_lcd_aux_data |= mode; \
		lcd_aux_write4bits();   
#define lcd_command(value)\
		lcd_select_addr();\
		g_lcd_aux_data = value; \
		g_lcd_aux_data &= 0xf0; \
		lcd_aux_write4bits();   \
		g_lcd_aux_data = value; \ 
		g_lcd_aux_data <<= 4;   \
		g_lcd_aux_data &= 0xf0; \
		lcd_aux_write4bits();   

// When the display powers up, it is configured as follows:
//
// 1. Display clear
// 2. Function set: 
//    DL = 1; 8-bit interface data 
//    N = 0; 1-line display 
//    F = 0; 5x8 dot character font 
// 3. Display on/off control: 
//    D = 0; Display off 
//    C = 0; Cursor off 
//    B = 0; Blinking off 
// 4. Entry mode set: 
//    I/D = 1; Increment by 1
//    S = 0; No shift 
//
// Note, however, that resetting the Arduino doesn't reset the LCD, so we
// can't assume that its in that state when a sketch starts (and the
// LiquidCrystal constructor is called).

ZUNO_LCD_I2C::ZUNO_LCD_I2C(byte lcd_Addr,byte lcd_cols,byte lcd_rows)
{
  _Addr = lcd_Addr;
  _cols = lcd_cols;
  _rows = lcd_rows;
  _backlightval = LCD_NOBACKLIGHT;
}

void ZUNO_LCD_I2C::init(){
	Wire.begin();
	_displayfunction = LCD_4BITMODE | LCD_1LINE | LCD_5x8DOTS;
	begin(_cols, _rows);  
}


void ZUNO_LCD_I2C::begin(byte cols, byte lines, byte dotsize) {
	if (lines > 1) {
		_displayfunction |= LCD_2LINE;
	}
	_numlines = lines;

	// for some 1 line displays you can select a 10 pixel high font
	if ((dotsize != 0) && (lines == 1)) {
		_displayfunction |= LCD_5x10DOTS;
	}

	// SEE PAGE 45/46 FOR INITIALIZATION SPECIFICATION!
	// according to datasheet, we need at least 40ms after power rises above 2.7V
	// before sending commands. Arduino can turn on way befer 4.5V so we'll wait 50
	delay(50); 
  	lcd_select_addr();
	// Now we pull both RS and R/W low to begin commands
	lcd_expander_write(_backlightval);	// reset expanderand turn backlight off (Bit 8 =1)
	delay(1000);

  	//put the LCD into 4 bit mode
	// this is according to the hitachi HD44780 datasheet
	// figure 24, pg 46
	
	  // we start in 8bit mode, try to set 4 bit mode
   lcd_write4bits(0x03 << 4);
   delayMicroseconds(4500); // wait min 4.1ms
   
   // second try
   lcd_write4bits(0x03 << 4);
   delayMicroseconds(4500); // wait min 4.1ms
   
   // third go!
   lcd_write4bits(0x03 << 4); 
   delayMicroseconds(150);
   
   // finally, set to 4-bit interface
   lcd_write4bits(0x02 << 4); 


	// set # lines, font size, etc.
	lcd_command(LCD_FUNCTIONSET | (_displayfunction));  
	
	// turn the display on with no cursor or blinking default
	_displaycontrol = LCD_DISPLAYON | LCD_CURSOROFF | LCD_BLINKOFF;
	display();
	
	// clear it off
	clear();
	
	// Initialize to default text direction (for roman languages)
	_displaymode = LCD_ENTRYLEFT | LCD_ENTRYSHIFTDECREMENT;
	
	// set the entry mode
	lcd_command(LCD_ENTRYMODESET | (_displaymode));
	
	home();
  
}
void ZUNO_LCD_I2C::write(byte value) {
	lcd_send(value, Rs);
}
/********** high level commands, for the user! */
void ZUNO_LCD_I2C::clear(){
	lcd_command(LCD_CLEARDISPLAY);// clear display, set cursor position to zero
	delayMicroseconds(2000);  // this command takes a long time!
}

void ZUNO_LCD_I2C::home(){
	lcd_command(LCD_RETURNHOME);  // set cursor position to zero
	delayMicroseconds(2000);  // this command takes a long time!
}

void ZUNO_LCD_I2C::bottomline(){
	lcd_command(LCD_SETDDRAMADDR | (0+0x54));
}	

void ZUNO_LCD_I2C::setCursor(byte col, byte row){
	
	lcd_command(LCD_SETDDRAMADDR | (col + g_lcd_row_offsets[row]));
}

// Turn the display on/off (quickly)
void ZUNO_LCD_I2C::noDisplay() {
	_displaycontrol &= ~LCD_DISPLAYON;
	lcd_command(LCD_DISPLAYCONTROL | (_displaycontrol));
}
void ZUNO_LCD_I2C::display() {
	_displaycontrol |= LCD_DISPLAYON;
	lcd_command(LCD_DISPLAYCONTROL | (_displaycontrol));
}

// Turns the underline cursor on/off
void ZUNO_LCD_I2C::noCursor() {
	_displaycontrol &= ~LCD_CURSORON;
	lcd_command(LCD_DISPLAYCONTROL | (_displaycontrol));
}
void ZUNO_LCD_I2C::cursor() {
	_displaycontrol |= LCD_CURSORON;
	lcd_command(LCD_DISPLAYCONTROL | (_displaycontrol));
}

// Turn on and off the blinking cursor
void ZUNO_LCD_I2C::noBlink() {
	_displaycontrol &= ~LCD_BLINKON;
	lcd_command(LCD_DISPLAYCONTROL | (_displaycontrol));
}
void ZUNO_LCD_I2C::blink() {
	_displaycontrol |= LCD_BLINKON;
	lcd_command(LCD_DISPLAYCONTROL | (_displaycontrol));
}

// These commands scroll the display without changing the RAM
void ZUNO_LCD_I2C::scrollDisplayLeft(void) {
	lcd_command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVELEFT);
}
void ZUNO_LCD_I2C::scrollDisplayRight(void) {
	lcd_command(LCD_CURSORSHIFT | LCD_DISPLAYMOVE | LCD_MOVERIGHT);
}

// This is for text that flows Left to Right
void ZUNO_LCD_I2C::leftToRight(void) {
	_displaymode |= LCD_ENTRYLEFT;
	lcd_command(LCD_ENTRYMODESET | (_displaymode));
}

// This is for text that flows Right to Left
void ZUNO_LCD_I2C::rightToLeft(void) {
	_displaymode &= ~LCD_ENTRYLEFT;
	lcd_command(LCD_ENTRYMODESET | (_displaymode));
}

// This will 'right justify' text from the cursor
void ZUNO_LCD_I2C::autoscroll(void) {
	_displaymode |= LCD_ENTRYSHIFTINCREMENT;
	lcd_command(LCD_ENTRYMODESET | (_displaymode));
}

// This will 'left justify' text from the cursor
void ZUNO_LCD_I2C::noAutoscroll(void) {
	_displaymode &= ~LCD_ENTRYSHIFTINCREMENT;
	lcd_command(LCD_ENTRYMODESET | (_displaymode));
}

// Allows us to fill the first 8 CGRAM locations
// with custom characters
void ZUNO_LCD_I2C::createChar(byte location, byte *charmap) {
	location &= 0x7; // we only have 8 locations 0-7
	lcd_command(LCD_SETCGRAMADDR | (location << 3));
	for (int i=0; i<8; i++) {
		write(*(charmap+i));
	}
}

// Turn the (optional) backlight off/on
void ZUNO_LCD_I2C::noBacklight(void) {
	_backlightval=LCD_NOBACKLIGHT;
	lcd_select_addr();
	lcd_expander_write(0);
}

void ZUNO_LCD_I2C::backlight(void) {
	_backlightval=LCD_BACKLIGHT;
	lcd_select_addr();
	lcd_expander_write(0);
}

// Alias functions

void ZUNO_LCD_I2C::cursor_on(){
	cursor();
}

void ZUNO_LCD_I2C::cursor_off(){
	noCursor();
}

void ZUNO_LCD_I2C::blink_on(){
	blink();
}

void ZUNO_LCD_I2C::blink_off(){
	noBlink();
}

void ZUNO_LCD_I2C::load_custom_character(byte char_num, byte *rows){
		createChar(char_num, rows); 
}

void ZUNO_LCD_I2C::setBacklight(byte new_val){
	if(new_val){
		backlight();		// turn backlight on
	}else{
		noBacklight();		// turn backlight off
	}
}

void ZUNO_LCD_I2C::printstr(char *c){
	print(c); 
}


// unsupported API functions
void ZUNO_LCD_I2C::off(){}
void ZUNO_LCD_I2C::on(){}
void ZUNO_LCD_I2C::setDelay (int cmdDelay,int charDelay) {}
byte ZUNO_LCD_I2C::status(){return 0;}
byte ZUNO_LCD_I2C::keypad (){return 0;}
byte ZUNO_LCD_I2C::init_bargraph(byte graphtype){return 0;}
void ZUNO_LCD_I2C::draw_horizontal_graph(byte row, byte column, byte len,  byte pixel_col_end){}
void ZUNO_LCD_I2C::draw_vertical_graph(byte row, byte column, byte len,  byte pixel_row_end){}
void ZUNO_LCD_I2C::setContrast(byte new_val){}

	
