//YWROBOT
#ifndef ZUNO_LCD_I2C_h
#define ZUNO_LCD_I2C_h

#include "Arduino.h"
#include "Print.h" 
#include "Wire.h"
#include "binary.h"

// flags for function set
#define LCD_8BITMODE 0x10
#define LCD_4BITMODE 0x00
#define LCD_2LINE 0x08
#define LCD_1LINE 0x00
#define LCD_5x10DOTS 0x04
#define LCD_5x8DOTS 0x00

class ZUNO_LCD_I2C : public Print {
public:
  ZUNO_LCD_I2C(byte lcd_Addr,byte lcd_cols,byte lcd_rows);
  void begin(byte cols, byte rows, byte charsize = LCD_5x8DOTS );
  void clear();
  void home();
  void noDisplay();
  void display();
  void noBlink();
  void blink();
  void noCursor();
  void cursor();
  void scrollDisplayLeft();
  void scrollDisplayRight();
  void printLeft();
  void printRight();
  void leftToRight();
  void rightToLeft();
  void shiftIncrement();
  void shiftDecrement();
  void noBacklight();
  void backlight();
  void autoscroll();
  void bottomline();
  void noAutoscroll(); 
  void createChar(byte, byte *);
  void setCursor(byte, byte); 
  virtual void write(byte);
  void init();

////compatibility API function aliases
void blink_on();						// alias for blink()
void blink_off();       					// alias for noBlink()
void cursor_on();      	 					// alias for cursor()
void cursor_off();      					// alias for noCursor()
void setBacklight(byte new_val);				// alias for backlight() and nobacklight()
void load_custom_character(byte char_num, byte *rows);	// alias for createChar()
void printstr(char *);

////Unsupported API functions (not implemented in this library)
byte status();
void setContrast(byte new_val);
byte keypad();
void setDelay(int,int);
void on();
void off();
byte init_bargraph(byte graphtype);
void draw_horizontal_graph(byte row, byte column, byte len,  byte pixel_col_end);
void draw_vertical_graph(byte row, byte column, byte len,  byte pixel_col_end);
	 

private:
  byte _Addr;
  byte _displayfunction;
  byte _displaycontrol;
  byte _displaymode;
  byte _numlines;
  byte _cols;
  byte _rows;
  byte _backlightval;
};

#endif
